@@ -2,31 +2,61 @@ import copy
 import re
 
 
+class CircleDict:
+    def __init__(self):
+        self.d = {}
+
+    def __getitem__(self, item):
+        return self.d[item]
+
+    def __setitem__(self, key, value):
+        self.d[key] = value
+
+    def __contains__(self, item):
+        return item in self.d
+
+
 class DictCompiler:
     p = re.compile('\\${(.*?)}')
+    pf = re.compile('\\${(.*?)}\\$')
 
     class InCircle:
         pass
 
+    class NotFound:
+        pass
+
     def __init__(self, d: dict):
         self.d = copy.deepcopy(d)
-        self.circle = dict()
+        self.circle = CircleDict()
 
     def parse(self):
-        return self._process_dict([], self.d)
+        return self._process([], self.d)
 
     def _get_value(self, path: str):
         path = path.split('.')
+        present_path = []
         v = self.d
         for k in path:
+            present_path.append(k)
+
             if isinstance(v, list) or isinstance(v, tuple):
                 k = int(k)
                 if len(v) < k:
-                    raise KeyError(f'array out of bound (index = {k}) when retrieving value where path is {path}')
-            else:
+                    raise KeyError(f'array out of bound (index = {k}) when retrieving value where path is {present_path}')
+            elif isinstance(v, dict):
                 if k not in v:
-                    raise KeyError(f'key ({k}) not exist when retrieving value where path is {path}')
+                    raise KeyError(f'key ({k}) not exist when retrieving value where path is {".".join(present_path)}')
+            elif isinstance(v, str):
+                raise KeyError(f'key ({k}) not exist when retrieving value where path is {".".join(present_path)}')
+
             v = v[k]
+
+            while isinstance(v, str):
+                full_match_value = self._process_full_ref('.'.join(present_path), v)
+                if full_match_value is self.NotFound:
+                    break
+                v = full_match_value
         return v
 
     @classmethod
@@ -54,6 +84,18 @@ class DictCompiler:
             return self._process_item('.'.join(path), v)
         return v
 
+    def _process_full_ref(self, path: str, s: str):
+        match = self.pf.fullmatch(s)
+        if not match:
+            return self.NotFound
+
+        full_ref = match.group(1)
+        self.circle[path] = self.InCircle()
+        value = self._get_value(full_ref)
+        value = self._process(path.split('.'), value)
+        self.circle[path] = value
+        return value
+
     def _process_item(self, path: str, s: str):
         if not isinstance(s, str):
             return s
@@ -64,12 +106,15 @@ class DictCompiler:
                 raise ValueError('Dict references are in circle')
             return v
 
+        full_value = self._process_full_ref(path, s)
+        if full_value is not self.NotFound:
+            return full_value
+
         refs = self._get_refs(s)
         if not refs:
             return s
 
         self.circle[path] = self.InCircle()
-
         parsed_s = []
         position = 0
         for r, p, v in self._parse(s, refs):
@@ -104,14 +149,15 @@ def parse(d: dict):
 
 if __name__ == '__main__':
     d = dict(
-        a='${b}+1',
-        b='asas${c.v.2}cool',
+        a='${b.v.1}+1',
+        b='${c}$',
         c=dict(
             __l=23,
             v=('sorry', 'good', 'ok'),
             m='${c.__l}'
         )
     )
+    # d = [1,2,'${0}$']
 
     rd = DictCompiler(d)
     print(rd.parse())
